﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using System.Xml;

namespace EduNetworkBuilder
{
    /// <summary>
    /// For drawing shapes on a network.  Rectangles, circles.  Filled or not
    /// </summary>
    public class NetShape
    {
        public NetShapeType MyShape = NetShapeType.none;
        public Rectangle InArea;
        public Color LineColor = Color.Empty;
        public Color FillColor = Color.Empty;
        int DragSize = 15;

        public NetShape(NetShapeType What, Rectangle Where, Color color)
        {
            MyShape = What;
            InArea = Where;
            FillColor = color;
        }
        public NetShape(XmlNode what)
        {
            Load(what);
        }

        /// <summary>
        /// Draw the specified shape on the image
        /// </summary>
        /// <param name="baseImage"></param>
        public void Draw(Image baseImage, bool InShapeEditMode = false)
        {
            Graphics G = Graphics.FromImage(baseImage);
            Pen trimPen = new Pen(LineColor);
            Brush coloredBrush = new SolidBrush(FillColor);
            switch(MyShape)
            {
                case NetShapeType.circle:
                    if(FillColor != Color.Empty)
                        G.FillEllipse(coloredBrush,InArea);
                    if (LineColor != Color.Empty)
                        G.DrawEllipse(trimPen, InArea);
                    break;
                case NetShapeType.rectangle:
                    if (FillColor != Color.Empty)
                        G.FillRectangle(coloredBrush, InArea);
                    if (LineColor != Color.Empty)
                        G.DrawRectangle(trimPen, InArea);
                    break;
            }
            if(InShapeEditMode)
            {
                int sz = DragSize;
                Color edgeColor = Color.Black;
                if (FillColor == Color.Black) edgeColor = Color.White;
                coloredBrush = new SolidBrush(edgeColor);
                //We want to put drag-marks on the corners of our rectangle
                foreach(Rectangle one in Corners())
                    G.FillRectangle(coloredBrush, one);
            }
            G.Dispose();
        }

        List<Rectangle> Corners()
        {
            int sz = DragSize;
            List<Rectangle> tCorners = new List<Rectangle>();
            Rectangle TL = new Rectangle(InArea.X, InArea.Y, sz, sz);
            Rectangle TR = new Rectangle(InArea.X + InArea.Width - sz, InArea.Y, sz, sz);
            Rectangle BL = new Rectangle(InArea.X, InArea.Y + InArea.Height - sz, sz, sz);
            Rectangle BR = new Rectangle(InArea.X + InArea.Width - sz, InArea.Y + InArea.Height - sz, sz, sz);
            tCorners.Add(TL);
            tCorners.Add(TR);
            tCorners.Add(BL);
            tCorners.Add(BR);
            return tCorners;
        }

        public void Load(XmlNode theNode)
        {
            foreach (XmlNode Individual in theNode.ChildNodes)
            {
                XmlNodeType myNodetype = Individual.NodeType;
                if (myNodetype == XmlNodeType.Element)
                {
                    switch (Individual.Name.ToLower())
                    {
                        case "what":
                            MyShape = NB.TryParseEnum<NetShapeType>(Individual.InnerText, NetShapeType.none);
                            break;
                        case "where":
                            RectangleConverter converter = new RectangleConverter();
                            InArea = (Rectangle)converter.ConvertFromString(Individual.InnerText);
                            break;
                        case "fillcolor":
                            FillColor = Color.FromName(Individual.InnerText);
                            break;
                        case "linecolor":
                            LineColor = Color.FromName(Individual.InnerText);
                            break;
                    }
                }
            }
        }

        public void Save(XmlWriter writer)
        {
            writer.WriteStartElement("shape");
            writer.WriteElementString("what", MyShape.ToString());
            RectangleConverter converter = new RectangleConverter();
            writer.WriteElementString("where", converter.ConvertToString(InArea));
            writer.WriteElementString("fillcolor", FillColor.Name);
            writer.WriteElementString("linecolor", LineColor.Name);
            writer.WriteEndElement();
        }
    }
}
